@@ -1,11 +1,14 @@
-use typed_builder::TypedBuilder;
-
+//! In-memory fuzzer with `QEMU`-based binary-only instrumentation
+//!
+use core::fmt::{self, Debug, Formatter};
 use std::{fs, net::SocketAddr, path::PathBuf, time::Duration};
+use typed_builder::TypedBuilder;
 
 use libafl::{
     bolts::{
         current_nanos,
         launcher::Launcher,
+        os::Cores,
         rands::StdRand,
         shmem::{ShMemProvider, StdShMemProvider},
         tuples::{tuple_list, Merge},
@@ -29,12 +32,14 @@ use libafl::{
     state::{HasCorpus, HasMetadata, StdState},
 };
 
-pub use libafl_qemu::emu;
+pub use libafl_qemu::emu::Emulator;
 use libafl_qemu::{cmplog, edges, QemuCmpLogHelper, QemuEdgeCoverageHelper, QemuExecutor};
 use libafl_targets::CmpLogObserver;
 
 use crate::{CORPUS_CACHE_SIZE, DEFAULT_TIMEOUT_SECS};
 
+/// Sugar to create a `libfuzzer`-style fuzzer that uses
+/// `QEMU`-based binary-only instrumentation
 #[derive(TypedBuilder)]
 pub struct QemuBytesCoverageSugar<'a, H>
 where
@@ -56,10 +61,12 @@ where
     /// Flag if use CmpLog
     #[builder(default = false)]
     use_cmplog: bool,
+    /// The port the fuzzing nodes communicate over
+    /// This will spawn a server on this port, and connect to other brokers using this port.
     #[builder(default = 1337_u16)]
     broker_port: u16,
     /// The list of cores to run on
-    cores: &'a [usize],
+    cores: &'a Cores,
     /// The `ip:port` address of another broker to connect our new broker to for multi-machine
     /// clusters.
     #[builder(default = None, setter(strip_option))]
@@ -69,12 +76,40 @@ where
     harness: Option<H>,
 }
 
+impl<'a, H> Debug for QemuBytesCoverageSugar<'a, H>
+where
+    H: FnMut(&[u8]),
+{
+    fn fmt(&self, f: &mut Formatter<'_>) -> fmt::Result {
+        f.debug_struct("QemuBytesCoverageSugar")
+            .field("configuration", &self.configuration)
+            .field("timeout", &self.timeout)
+            .field("input_dirs", &self.input_dirs)
+            .field("output_dir", &self.output_dir)
+            .field("tokens_file", &self.tokens_file)
+            .field("use_cmplog", &self.use_cmplog)
+            .field("broker_port", &self.broker_port)
+            .field("cores", &self.cores)
+            .field("remote_broker_addr", &self.remote_broker_addr)
+            .field(
+                "harness",
+                if self.harness.is_some() {
+                    &"<harness_fn>"
+                } else {
+                    &"None"
+                },
+            )
+            .finish()
+    }
+}
+
 impl<'a, H> QemuBytesCoverageSugar<'a, H>
 where
     H: FnMut(&[u8]),
 {
+    /// Run the fuzzer
     #[allow(clippy::too_many_lines, clippy::similar_names)]
-    pub fn run(&mut self) {
+    pub fn run(&mut self, emulator: &Emulator) {
         let conf = match self.configuration.as_ref() {
             Some(name) => EventConfig::from_name(name),
             None => EventConfig::AlwaysUnique,
@@ -171,6 +206,7 @@ where
             if self.use_cmplog {
                 let executor = QemuExecutor::new(
                     &mut harness,
+                    emulator,
                     tuple_list!(QemuEdgeCoverageHelper::new(), QemuCmpLogHelper::new()),
                     tuple_list!(edges_observer, time_observer),
                     &mut fuzzer,
@@ -246,6 +282,7 @@ where
             } else {
                 let executor = QemuExecutor::new(
                     &mut harness,
+                    emulator,
                     tuple_list!(QemuEdgeCoverageHelper::new()),
                     tuple_list!(edges_observer, time_observer),
                     &mut fuzzer,
@@ -327,9 +364,12 @@ where
     }
 }
 
+/// python bindings for this sugar
 #[cfg(feature = "python")]
 pub mod pybind {
     use crate::qemu;
+    use libafl::bolts::os::Cores;
+    use libafl_qemu::emu::pybind::Emulator;
     use pyo3::prelude::*;
     use pyo3::types::PyBytes;
     use std::path::PathBuf;
@@ -339,11 +379,12 @@ pub mod pybind {
         input_dirs: Vec<PathBuf>,
         output_dir: PathBuf,
         broker_port: u16,
-        cores: Vec<usize>,
+        cores: Cores,
     }
 
     #[pymethods]
     impl QemuBytesCoverageSugar {
+        /// Create a new [`QemuBytesCoverageSugar`]
         #[new]
         fn new(
             input_dirs: Vec<PathBuf>,
@@ -355,12 +396,13 @@ pub mod pybind {
                 input_dirs,
                 output_dir,
                 broker_port,
-                cores,
+                cores: cores.into(),
             }
         }
 
+        /// Run the fuzzer
         #[allow(clippy::needless_pass_by_value)]
-        pub fn run(&self, harness: PyObject) {
+        pub fn run(&self, emulator: &Emulator, harness: PyObject) {
             qemu::QemuBytesCoverageSugar::builder()
                 .input_dirs(&self.input_dirs)
                 .output_dir(self.output_dir.clone())
@@ -375,10 +417,11 @@ pub mod pybind {
                     .unwrap();
                 })
                 .build()
-                .run();
+                .run(&emulator.emu);
         }
     }
 
+    /// Register this class
     pub fn register(_py: Python, m: &PyModule) -> PyResult<()> {
         m.add_class::<QemuBytesCoverageSugar>()?;
         Ok(())
